@@ -10,6 +10,7 @@
 
 #include "ssl.h"
 #include "sslerr.h"
+#include "pk11hpke.h"
 
 SEC_BEGIN_PROTOS
 
@@ -502,21 +503,62 @@ typedef SECStatus(PR_CALLBACK *SSLResumptionTokenCallback)(
                          (PRFileDesc * _fd, PRUint32 _size), \
                          (fd, size))
 
-/* If |enabled|, a GREASE ECH extension will be sent in every ClientHello,
+/* Client:
+ * If |enabled|, a GREASE ECH extension will be sent in every ClientHello,
  * unless a valid and supported ECHConfig is configured to the socket
- * (in which case real ECH takes precedence). If |!enabled|, it is not sent.*/
+ * (in which case real ECH takes precedence). If |!enabled|, it is not sent.
+ *
+ * Server:
+ * If |enabled|, a GREASE ECH extensions will be sent in every HelloRetryRequest,
+ * provided that the corresponding ClientHello contained an ECH extension. If ECH
+ * is enabled, the real ECH HRR extension takes precedence.
+ */
 #define SSL_EnableTls13GreaseEch(fd, enabled)        \
     SSL_EXPERIMENTAL_API("SSL_EnableTls13GreaseEch", \
                          (PRFileDesc * _fd, PRBool _enabled), (fd, enabled))
 
-/* If |enabled|, a server receiving a Client Hello containing the ech_is_inner
- * (and not encrypted_client_hello) extension will respond with the ECH
+/*
+ * Client:
+ * When sending a GREASE ECH extension in a ClientHello, pad it as though the
+ * hypothetical ECHConfig had |maximum_name_length| equal to |size|. |size| may
+ * vary between 1 and 255 and defaults to 100.
+ *
+ * Server:
+ * Has no effect.
+ */
+#define SSL_SetTls13GreaseEchSize(fd, size)           \
+    SSL_EXPERIMENTAL_API("SSL_SetTls13GreaseEchSize", \
+                         (PRFileDesc * _fd, PRUint8 _size), (fd, size))
+
+/* If |enabled|, a server receiving a Client Hello containing an encrypted_client_hello
+ * of type inner will respond with the ECH
  * acceptance signal. This signals the client to continue with the inner
  * transcript rather than outer. */
 #define SSL_EnableTls13BackendEch(fd, enabled)        \
     SSL_EXPERIMENTAL_API("SSL_EnableTls13BackendEch", \
                          (PRFileDesc * _fd, PRBool _enabled), (fd, enabled))
 
+/* This allows an extension writer to supply different values for inner and
+ * outer ClientHello when using encrypted ClientHello.
+ *
+ * When enabled, each extension writer can be called more than once for the same
+ * message; it must provide the same response when called for the same message
+ * type.  When calling the writer to construct the outer ClientHello, the
+ * function will be called with ssl_hs_ech_outer_client_hello as the message
+ * type (a value from outside the range of valid TLS handshake messages).
+ *
+ * When disabled, the extension writer is called once for the outer ClientHello
+ * and the value is copied to the inner ClientHello.
+ *
+ * Enabling this affects all extension writers.  The order in which extension
+ * writers are added is also important.  Any extension writer that writes
+ * different values for inner and outer ClientHello will prevent later
+ * extensions from being compressed.
+ */
+#define SSL_CallExtensionWriterOnEchInner(fd, enabled)        \
+    SSL_EXPERIMENTAL_API("SSL_CallExtensionWriterOnEchInner", \
+                         (PRFileDesc * _fd, PRBool _enabled), (fd, enabled))
+
 /* Called by the client after an initial ECH connection fails with
  * SSL_ERROR_ECH_RETRY_WITH_ECH. Returns compatible ECHConfigs, which
  * are configured via SetClientEchConfigs for an ECH retry attempt.
@@ -566,30 +608,33 @@ typedef SECStatus(PR_CALLBACK *SSLResumptionTokenCallback)(
 /*
  * Generate an encoded ECHConfig structure (presumably server side).
  *
+ * configId -- an identifier for the configuration.
  * publicName -- the public_name value to be placed in SNI.
+ * maxNameLen -- the maximum length of protected names
+ * kemId -- the HKPE KEM ID value
+ * pubKey -- the public key for the key pair
  * hpkeSuites -- the HPKE cipher suites that can be used
  * hpkeSuitesCount -- the number of suites in hpkeSuites
- * kemId -- the HKPE KEM ID value
- * group -- the named group this key corresponds to
- * pubKey -- the public key for the key pair
- * pad -- the maximum length to pad to
  * out/outlen/maxlen -- where to output the data
  */
-#define SSL_EncodeEchConfig(publicName, hpkeSuites, hpkeSuitesCount, \
-                            kemId, pubKey, maxNameLen, out, outlen,  \
-                            maxlen)                                  \
-    SSL_EXPERIMENTAL_API("SSL_EncodeEchConfig",                      \
-                         (const char *_publicName,                   \
-                          const PRUint32 *_hpkeSuites,               \
-                          unsigned int _hpkeSuitesCount,             \
-                          HpkeKemId _kemId,                          \
-                          const SECKEYPublicKey *_pubKey,            \
-                          PRUint16 _maxNameLen,                      \
-                          PRUint8 *_out, unsigned int *_outlen,      \
-                          unsigned int _maxlen),                     \
-                         (publicName, hpkeSuites, hpkeSuitesCount,   \
-                          kemId, pubKey, maxNameLen, out, outlen,    \
-                          maxlen))
+typedef struct HpkeSymmetricSuiteStr {
+    HpkeKdfId kdfId;
+    HpkeAeadId aeadId;
+} HpkeSymmetricSuite;
+#define SSL_EncodeEchConfigId(configId, publicName, maxNameLen,          \
+                              kemId, pubKey, hpkeSuites, hpkeSuiteCount, \
+                              out, outlen, maxlen)                       \
+    SSL_EXPERIMENTAL_API("SSL_EncodeEchConfigId",                        \
+                         (PRUint8 _configId, const char *_publicName,    \
+                          unsigned int _maxNameLen, HpkeKemId _kemId,    \
+                          const SECKEYPublicKey *_pubKey,                \
+                          const HpkeSymmetricSuite *_hpkeSuites,         \
+                          unsigned int _hpkeSuiteCount,                  \
+                          PRUint8 *_out, unsigned int *_outlen,          \
+                          unsigned int _maxlen),                         \
+                         (configId, publicName, maxNameLen,              \
+                          kemId, pubKey, hpkeSuites, hpkeSuiteCount,     \
+                          out, outlen, maxlen))
 
 /* SSL_SetSecretCallback installs a callback that TLS calls when it installs new
  * traffic secrets.
@@ -1028,6 +1073,18 @@ typedef struct SSLMaskingContextStr {
                           unsigned int _identityLen),                 \
                          (fd, identity, identityLen))
 
+/* The next function is used to provide support for TLS RFC 8879 
+ * (Certificate Compression). 
+ * 
+ * The function SSL_SetCertificateCompressionAlgorithm() adds a certificate
+ * compression mechanism to the socket fd. */
+
+#define SSL_SetCertificateCompressionAlgorithm(fd, t)                \
+    SSL_EXPERIMENTAL_API("SSL_SetCertificateCompressionAlgorithm",   \
+                         (PRFileDesc * _fd,                          \
+                          SSLCertificateCompressionAlgorithmType t), \
+                         (fd, t))
+
 /* Deprecated experimental APIs */
 #define SSL_UseAltServerHelloType(fd, enable) SSL_DEPRECATED_EXPERIMENTAL_API
 #define SSL_SetupAntiReplay(a, b, c) SSL_DEPRECATED_EXPERIMENTAL_API
@@ -1035,6 +1092,7 @@ typedef struct SSLMaskingContextStr {
 #define SSL_EnableESNI(a, b, c, d) SSL_DEPRECATED_EXPERIMENTAL_API
 #define SSL_EncodeESNIKeys(a, b, c, d, e, f, g, h, i, j) SSL_DEPRECATED_EXPERIMENTAL_API
 #define SSL_SetESNIKeyPair(a, b, c, d) SSL_DEPRECATED_EXPERIMENTAL_API
+#define SSL_EncodeEchConfig(a, b, c, d, e, f, g, h, i) SSL_DEPRECATED_EXPERIMENTAL_API
 
 SEC_END_PROTOS
 
